From e2e3a60fcc42eea4703c6f7b2877eae7188a73bc Mon Sep 17 00:00:00 2001
From: =?UTF-8?q?Mattias=20Engdeg=C3=A5rd?= <mattiase@acm.org>
Date: Fri, 11 Oct 2019 14:42:48 +0200
Subject: [PATCH] Make undo confinement to active region user-settable
 (bug#37700)

* lisp/simple.el (undo-confined-to-active-region, undo):
Add defcustom `undo-confined-to-active-region' that controls whether
the effects of `undo' are confined to the region when it is active.
The default value is t.
---
 lisp/simple.el | 15 +++++++++++++--
 1 file changed, 13 insertions(+), 2 deletions(-)

diff --git a/lisp/simple.el b/lisp/simple.el
index 597278ae2b..10dec26200 100644
--- a/lisp/simple.el
+++ b/lisp/simple.el
@@ -2464,6 +2464,14 @@ minibuffer-error-function
 ;Put this on C-x u, so we can force that rather than C-_ into startup msg
 (define-obsolete-function-alias 'advertised-undo 'undo "23.2")
 
+(defcustom undo-confined-to-active-region t
+  "If non-nil, the effects of `undo' without argument are confined to
+the active region when there is one.  If nil, the effects of `undo'
+are independent of the region."
+  :type 'boolean
+  :group 'undo
+  :version "27.1")
+
 (defconst undo-equiv-table (make-hash-table :test 'eq :weakness t)
   "Table mapping redo records to the corresponding undo one.
 A redo record for undo-in-region maps to t.
@@ -2485,7 +2493,9 @@ undo
 A numeric ARG serves as a repeat count.
 
 In Transient Mark mode when the mark is active, only undo changes within
-the current region.  Similarly, when not in Transient Mark mode, just \\[universal-argument]
+the current region; this behaviour can be changed by customizing
+the variable `undo-confined-to-active-region'.
+Similarly, when not in Transient Mark mode, just \\[universal-argument]
 as an argument limits undo to changes within the current region."
   (interactive "*P")
   ;; Make last-command indicate for the next command that this was an undo.
@@ -2517,7 +2527,8 @@ undo
 		       ;; it shows nothing else happened in between.
 		       (gethash list undo-equiv-table))))
       (setq undo-in-region
-	    (or (region-active-p) (and arg (not (numberp arg)))))
+	    (or (and (region-active-p) undo-confined-to-active-region)
+                (and arg (not (numberp arg)))))
       (if undo-in-region
 	  (undo-start (region-beginning) (region-end))
 	(undo-start))
-- 
2.21.0 (Apple Git-122)

